(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// lib/db-deprecated.js                                                //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
// This file imports symbols from the sandstorm-db package. New code should use SandstormDb
// directly.                                                           //
                                                                       //
// TODO(cleanup): Over time, eliminate the use of each of these assignments by using `globalDb`
//   directly. For collections, prefer to update SandstormDb to provide methods for querying the
//   collection rather than use `globalDb.collections` directly. When code is moved into packages,
//   the `globalDb` global should NOT go with it; the package should expect a SandstormDb to be
//   passed in, thus allowing mocking the database for unit tests.     //
                                                                       //
globalDb = new SandstormDb();                                          // 26
                                                                       //
Packages = globalDb.collections.packages;                              // 28
DevPackages = globalDb.collections.devPackages;                        // 29
UserActions = globalDb.collections.userActions;                        // 30
Grains = globalDb.collections.grains;                                  // 31
Contacts = globalDb.collections.contacts;                              // 32
Sessions = globalDb.collections.sessions;                              // 33
SignupKeys = globalDb.collections.signupKeys;                          // 34
ActivityStats = globalDb.collections.activityStats;                    // 35
DeleteStats = globalDb.collections.deleteStats;                        // 36
FileTokens = globalDb.collections.fileTokens;                          // 37
ApiTokens = globalDb.collections.apiTokens;                            // 38
Notifications = globalDb.collections.notifications;                    // 39
StatsTokens = globalDb.collections.statsTokens;                        // 40
Misc = globalDb.collections.misc;                                      // 41
Settings = globalDb.collections.settings;                              // 42
                                                                       //
currentUserGrains = globalDb.currentUserGrains.bind(globalDb);         // 44
isDemoUser = globalDb.isDemoUser.bind(globalDb);                       // 45
isSignedUp = globalDb.isSignedUp.bind(globalDb);                       // 46
isSignedUpOrDemo = globalDb.isSignedUpOrDemo.bind(globalDb);           // 47
isUserOverQuota = globalDb.isUserOverQuota.bind(globalDb);             // 48
isUserExcessivelyOverQuota = globalDb.isUserExcessivelyOverQuota.bind(globalDb);
isAdmin = globalDb.isAdmin.bind(globalDb);                             // 50
isAdminById = globalDb.isAdminById.bind(globalDb);                     // 51
findAdminUserForToken = globalDb.findAdminUserForToken.bind(globalDb);
matchWildcardHost = globalDb.matchWildcardHost.bind(globalDb);         // 53
makeWildcardHost = globalDb.makeWildcardHost.bind(globalDb);           // 54
allowDevAccounts = globalDb.allowDevAccounts.bind(globalDb);           // 55
roleAssignmentPattern = globalDb.roleAssignmentPattern;                // 56
                                                                       //
if (Meteor.isServer) {                                                 // 58
  getWildcardOrigin = globalDb.getWildcardOrigin.bind(globalDb);       // 59
                                                                       //
  Meteor.onConnection(function (connection) {                          // 61
    // TODO(cleanup): This is the best way I've thought of so far to allow methods declared in
    //   packages to actually use the DB, but it's pretty sad.         //
    connection.sandstormDb = globalDb;                                 // 64
  });                                                                  //
  SandstormDb.periodicCleanup(5 * 60 * 1000, SandstormPermissions.cleanupSelfDestructing(globalDb));
  SandstormDb.periodicCleanup(24 * 60 * 60 * 1000, function () {       // 67
    SandstormAutoupdateApps.updateAppIndex(globalDb);                  // 68
  });                                                                  //
                                                                       //
  Meteor.startup(function () {                                         // 71
    globalDb.migrateToLatest();                                        // 71
  });                                                                  //
  LDAP_DEFAULTS.url = globalDb.getLdapUrl();                           // 72
  LDAP_DEFAULTS.base = globalDb.getLdapBase();                         // 73
}                                                                      //
                                                                       //
if (Meteor.isClient) {                                                 // 76
  Session.setDefault("shrink-navbar", false);                          // 77
  globalGrains = new GrainViewList(globalDb);                          // 78
                                                                       //
  // If Meteor._localStorage disappears, we'll have to write our own localStorage wrapper, I guess.
  // Using window.localStorage is dangerous because it throws an exception if cookies are disabled.
  Session.set("shrink-navbar", Meteor._localStorage.getItem("shrink-navbar") === "true");
  globalTopbar = new SandstormTopbar(globalDb, {                       // 83
    get: function () {                                                 // 85
      return Session.get("topbar-expanded");                           // 86
    },                                                                 //
                                                                       //
    set: function (value) {                                            // 89
      Session.set("topbar-expanded", value);                           // 90
    }                                                                  //
  }, globalGrains, {                                                   //
    get: function () {                                                 // 95
      return Session.get("shrink-navbar");                             // 96
    },                                                                 //
                                                                       //
    set: function (value) {                                            // 99
      Meteor._localStorage.setItem("shrink-navbar", value);            // 100
      Session.set("shrink-navbar", value);                             // 101
    }                                                                  //
  });                                                                  //
                                                                       //
  globalAccountsUi = new AccountsUi(globalDb);                         // 105
                                                                       //
  Template.registerHelper("globalTopbar", function () {                // 107
    return globalTopbar;                                               // 107
  });                                                                  //
  Template.registerHelper("globalAccountsUi", function () {            // 108
    return globalAccountsUi;                                           // 108
  });                                                                  //
} else {                                                               //
  // TODO(cleanup): Refactor accounts registration stuff so that this doesn't need to be defined
  //   at all on the server.                                           //
  globalAccountsUi = null;                                             // 112
}                                                                      //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=db-deprecated.js.map
