(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/admin-server.js                                              //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var ADMIN_TOKEN_EXPIRATION_TIME = 15 * 60 * 1000;                      // 17
var publicAdminSettings = ["google", "github", "ldap", "saml", "emailToken", "splashUrl", "signupDialog", "adminAlert", "adminAlertTime", "adminAlertUrl", "termsUrl", "privacyUrl", "appMarketUrl", "appIndexUrl", "appUpdatesEnabled", "serverTitle", "returnAddress", "ldapNameField", "organizationMembership", "organizationSettings"];
                                                                       //
var FEATURE_KEY_FIELDS_PUBLISHED_TO_ADMINS = ["customer", "expires", "features", "isElasticBilling", "isTrial", "issued", "userLimit"];
                                                                       //
var PUBLIC_FEATURE_KEY_FIELDS = ["expires", "features"];               // 30
                                                                       //
var Fs = Npm.require("fs");                                            // 34
var Crypto = Npm.require("crypto");                                    // 35
var SANDSTORM_ADMIN_TOKEN = SANDSTORM_VARDIR + "/adminToken";          // 36
                                                                       //
var tokenIsValid = function (token) {                                  // 38
  if (token && Fs.existsSync(SANDSTORM_ADMIN_TOKEN)) {                 // 39
    var stats = Fs.statSync(SANDSTORM_ADMIN_TOKEN);                    // 40
    var expireTime = new Date(Date.now() - ADMIN_TOKEN_EXPIRATION_TIME);
    if (stats.mtime < expireTime) {                                    // 42
      return false;                                                    // 43
    } else {                                                           //
      return Fs.readFileSync(SANDSTORM_ADMIN_TOKEN, { encoding: "utf8" }) === token;
    }                                                                  //
  } else {                                                             //
    return false;                                                      // 48
  }                                                                    //
};                                                                     //
                                                                       //
var tokenIsSetupSession = function (token) {                           // 52
  if (token) {                                                         // 53
    var setupSession = globalDb.collections.setupSession.findOne({ _id: "current-session" });
    if (setupSession) {                                                // 55
      var hash = Crypto.createHash("sha256").update(token).digest("base64");
      var now = new Date();                                            // 57
      var sessionLifetime = 24 * 60 * 60 * 1000; // length of setup session validity, in milliseconds: 1 day
      if (setupSession.hashedSessionId === hash && now - setupSession.creationDate < sessionLifetime) {
        return true;                                                   // 60
      }                                                                //
    }                                                                  //
  }                                                                    //
                                                                       //
  return false;                                                        // 65
};                                                                     //
                                                                       //
// Export for use in other admin/ routes                               //
checkAuth = function (token) {                                         // 69
  check(token, Match.OneOf(undefined, null, String));                  // 70
  if (!isAdmin() && !tokenIsValid(token) && !tokenIsSetupSession(token)) {
    throw new Meteor.Error(403, "User must be admin or provide a valid token");
  }                                                                    //
};                                                                     //
                                                                       //
var clearAdminToken = function (token) {                               // 76
  if (tokenIsSetupSession(token)) {                                    // 77
    var hash = Crypto.createHash("sha256").update(token).digest("base64");
    globalDb.collections.setupSession.remove({                         // 79
      _id: "current-session",                                          // 80
      hashedSessionId: hash                                            // 81
    });                                                                //
  }                                                                    //
                                                                       //
  if (tokenIsValid(token)) {                                           // 85
    Fs.unlinkSync(SANDSTORM_ADMIN_TOKEN);                              // 86
    console.log("Admin token deleted.");                               // 87
  }                                                                    //
};                                                                     //
                                                                       //
var smtpConfigShape = {                                                // 91
  hostname: String,                                                    // 92
  port: Number,                                                        // 93
  auth: {                                                              // 94
    user: String,                                                      // 95
    pass: String                                                       // 96
  },                                                                   //
  returnAddress: String                                                // 98
};                                                                     //
                                                                       //
Meteor.methods({                                                       // 101
  setAccountSetting: function (token, serviceName, value) {            // 102
    checkAuth(token);                                                  // 103
    check(serviceName, String);                                        // 104
    check(value, Boolean);                                             // 105
                                                                       //
    // TODO(someday): currently this relies on the fact that an account is tied to a single
    // identity, and thus has only that entry in "services". This will need to be looked at when
    // multiple login methods/identities are allowed for a single account.
    if (!value && !tokenIsValid(token) && !tokenIsSetupSession(token) && serviceName in Meteor.user().services) {
      throw new Meteor.Error(403, "You can not disable the login service that your account uses.");
    }                                                                  //
                                                                       //
    // Only check configurations for OAuth services.                   //
    var oauthServices = ["google", "github"];                          // 116
    if (value && oauthServices.indexOf(serviceName) != -1) {           // 117
      var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
      var config = ServiceConfiguration.configurations.findOne({ service: serviceName });
      if (!config) {                                                   // 120
        throw new Meteor.Error(403, "You must configure the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
      }                                                                //
                                                                       //
      if (!config.clientId || !config.secret) {                        // 125
        throw new Meteor.Error(403, "You must provide a non-empty clientId and secret for the " + serviceName + " service before you can enable it. Click the \"configure\" link.");
      }                                                                //
    }                                                                  //
                                                                       //
    Settings.upsert({ _id: serviceName }, { $set: { value: value } });
    if (value) {                                                       // 132
      Settings.update({ _id: serviceName }, { $unset: { automaticallyReset: 1 } });
    }                                                                  //
  },                                                                   //
                                                                       //
  setSmtpConfig: function (token, config) {                            // 137
    checkAuth(token);                                                  // 138
    check(config, smtpConfigShape);                                    // 139
                                                                       //
    Settings.upsert({ _id: "smtpConfig" }, { $set: { value: config } });
  },                                                                   //
                                                                       //
  setSetting: function (token, name, value) {                          // 144
    checkAuth(token);                                                  // 145
    check(name, String);                                               // 146
    check(value, Match.OneOf(null, String, Date, Boolean));            // 147
                                                                       //
    Settings.upsert({ _id: name }, { $set: { value: value } });        // 149
  },                                                                   //
                                                                       //
  submitFeatureKey: function (token, textBlock) {                      // 152
    checkAuth(token);                                                  // 153
    check(textBlock, Match.OneOf(null, String));                       // 154
                                                                       //
    var db = this.connection.sandstormDb;                              // 156
                                                                       //
    if (!textBlock) {                                                  // 158
      // Delete the feature key.                                       //
      db.collections.featureKey.remove("currentFeatureKey");           // 160
      return;                                                          // 161
    }                                                                  //
                                                                       //
    // textBlock is a base64'd string, possibly with newlines and comment lines starting with "-"
    var featureKeyBase64 = _.chain(textBlock.split("\n")).filter(function (line) {
      return line.length > 0 && line[0] !== "-";                       //
    }).value().join("");                                               //
                                                                       //
    var buf = new Buffer(featureKeyBase64, "base64");                  // 170
    if (buf.length < 64) {                                             // 171
      throw new Meteor.Error(401, "Invalid feature key");              // 172
    }                                                                  //
                                                                       //
    // loadSignedFeatureKey is provided in feature-key.js              //
    var featureKey = loadSignedFeatureKey(buf);                        // 176
    if (!featureKey) {                                                 // 177
      throw new Meteor.Error(401, "Invalid feature key");              // 178
    }                                                                  //
                                                                       //
    // Persist the feature key in the database.                        //
    db.collections.featureKey.upsert("currentFeatureKey", {            // 182
      _id: "currentFeatureKey",                                        // 185
      value: buf                                                       // 186
    });                                                                //
  },                                                                   //
                                                                       //
  saveOrganizationSettings: function (token, params) {                 // 191
    checkAuth(token);                                                  // 192
    check(params, {                                                    // 193
      membership: {                                                    // 194
        emailToken: {                                                  // 195
          enabled: Boolean,                                            // 196
          domain: String                                               // 197
        },                                                             //
        google: {                                                      // 199
          enabled: Boolean,                                            // 200
          domain: String                                               // 201
        },                                                             //
        ldap: {                                                        // 203
          enabled: Boolean                                             // 204
        },                                                             //
        saml: {                                                        // 206
          enabled: Boolean                                             // 207
        }                                                              //
      },                                                               //
      settings: {                                                      // 210
        disallowGuests: Boolean                                        // 211
      }                                                                //
    });                                                                //
                                                                       //
    this.connection.sandstormDb.collections.settings.upsert({ _id: "organizationMembership" }, { value: params.membership });
    this.connection.sandstormDb.collections.settings.upsert({ _id: "organizationSettings" }, { value: params.settings });
  },                                                                   //
                                                                       //
  adminConfigureLoginService: function (token, options) {              // 219
    checkAuth(token);                                                  // 220
    check(options, Match.ObjectIncluding({ service: String }));        // 221
                                                                       //
    var ServiceConfiguration = Package["service-configuration"].ServiceConfiguration;
                                                                       //
    ServiceConfiguration.configurations.upsert({ service: options.service }, options);
  },                                                                   //
                                                                       //
  clearResumeTokensForService: function (token, serviceName) {         // 228
    checkAuth(token);                                                  // 229
    check(serviceName, String);                                        // 230
                                                                       //
    var query = {};                                                    // 232
    query["services." + serviceName] = { $exists: true };              // 233
    Meteor.users.find(query).forEach(function (identity) {             // 234
      if (identity.services.resume && identity.services.resume.loginTokens && identity.services.resume.loginTokens.length > 0) {
        Meteor.users.update({ _id: identity._id }, { $set: { "services.resume.loginTokens": [] } });
      }                                                                //
                                                                       //
      Meteor.users.update({ "loginIdentities.id": identity._id }, { $set: { "services.resume.loginTokens": [] } });
    });                                                                //
  },                                                                   //
                                                                       //
  adminUpdateUser: function (token, userInfo) {                        // 245
    checkAuth(token);                                                  // 246
    check(userInfo, {                                                  // 247
      userId: String,                                                  // 248
      signupKey: Boolean,                                              // 249
      isAdmin: Boolean                                                 // 250
    });                                                                //
                                                                       //
    var userId = userInfo.userId;                                      // 253
    if (userId === Meteor.userId() && !userInfo.isAdmin) {             // 254
      throw new Meteor.Error(403, "User cannot remove admin permissions from itself.");
    }                                                                  //
                                                                       //
    Meteor.users.update({ _id: userId }, { $set: _.omit(userInfo, ["_id", "userId"]) });
  },                                                                   //
                                                                       //
  testSend: function (token, smtpConfig, to) {                         // 261
    checkAuth(token);                                                  // 262
    check(smtpConfig, smtpConfigShape);                                // 263
    check(to, String);                                                 // 264
    var returnAddress = smtpConfig.returnAddress;                      //
    var restConfig = babelHelpers.objectWithoutProperties(smtpConfig, ["returnAddress"]);
                                                                       //
    SandstormEmail.send({                                              // 267
      to: to,                                                          // 268
      from: globalDb.getServerTitle() + " <" + returnAddress + ">",    // 269
      subject: "Testing your Sandstorm's SMTP setting",                // 270
      text: "Success! Your outgoing SMTP is working.",                 // 271
      smtpConfig: restConfig                                           // 272
    });                                                                //
  },                                                                   //
                                                                       //
  createSignupKey: function (token, note, quota) {                     // 276
    checkAuth(token);                                                  // 277
    check(note, String);                                               // 278
    check(quota, Match.OneOf(undefined, null, Number));                // 279
                                                                       //
    var key = Random.id();                                             // 281
    var content = { _id: key, used: false, note: note };               // 282
    if (typeof quota === "number") content.quota = quota;              // 283
    SignupKeys.insert(content);                                        // 284
    return key;                                                        // 285
  },                                                                   //
                                                                       //
  sendInvites: function (token, origin, from, list, subject, message, quota) {
    checkAuth(token);                                                  // 289
    check([origin, from, list, subject, message], [String]);           // 290
    check(quota, Match.OneOf(undefined, null, Number));                // 291
                                                                       //
    if (!from.trim()) {                                                // 293
      throw new Meteor.Error(403, "Must enter 'from' address.");       // 294
    }                                                                  //
                                                                       //
    if (!list.trim()) {                                                // 297
      throw new Meteor.Error(403, "Must enter 'to' addresses.");       // 298
    }                                                                  //
                                                                       //
    this.unblock();                                                    // 301
                                                                       //
    list = list.split("\n");                                           // 303
    for (var i in babelHelpers.sanitizeForInObject(list)) {            // 304
      var email = list[i].trim();                                      // 305
                                                                       //
      if (email) {                                                     // 307
        var key = Random.id();                                         // 308
                                                                       //
        var content = {                                                // 310
          _id: key,                                                    // 311
          used: false,                                                 // 312
          note: "E-mail invite to " + email,                           // 313
          email: email,                                                // 314
          definitelySent: false                                        // 315
        };                                                             //
        if (typeof quota === "number") content.quota = quota;          // 317
        SignupKeys.insert(content);                                    // 318
        SandstormEmail.send({                                          // 319
          to: email,                                                   // 320
          from: from,                                                  // 321
          envelopeFrom: globalDb.getReturnAddress(),                   // 322
          subject: subject,                                            // 323
          text: message.replace(/\$KEY/g, origin + "/signup/" + key)   // 324
        });                                                            //
        SignupKeys.update(key, { $set: { definitelySent: true } });    // 326
      }                                                                //
    }                                                                  //
                                                                       //
    return { sent: true };                                             // 330
  },                                                                   //
                                                                       //
  offerIpNetwork: function (token) {                                   // 333
    checkAuth(token);                                                  // 334
    if (!isAdmin()) {                                                  // 335
      throw new Meteor.Error(403, "Offering IpNetwork is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
    }                                                                  //
                                                                       //
    var requirements = [{ userIsAdmin: Meteor.userId() }];             // 340
    var sturdyRef = waitPromise(saveFrontendRef({ ipNetwork: true }, { webkey: null }, requirements)).sturdyRef;
    return ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef;
  },                                                                   //
                                                                       //
  offerIpInterface: function (token) {                                 // 349
    checkAuth(token);                                                  // 350
    if (!isAdmin()) {                                                  // 351
      throw new Meteor.Error(403, "Offering IpInterface is only allowed for logged in users " + "(a token is not sufficient). Please sign in with an admin account");
    }                                                                  //
                                                                       //
    var requirements = [{ userIsAdmin: Meteor.userId() }];             // 356
    var sturdyRef = waitPromise(saveFrontendRef({ ipInterface: true }, { webkey: null }, requirements)).sturdyRef;
    return ROOT_URL.protocol + "//" + globalDb.makeApiHost(sturdyRef) + "#" + sturdyRef;
  },                                                                   //
                                                                       //
  adminToggleDisableCap: function (token, capId, value) {              // 365
    checkAuth(token);                                                  // 366
    check(capId, String);                                              // 367
    check(value, Boolean);                                             // 368
                                                                       //
    if (value) {                                                       // 370
      ApiTokens.update({ _id: capId }, { $set: { revoked: true } });   // 371
    } else {                                                           //
      ApiTokens.update({ _id: capId }, { $set: { revoked: false } });  // 373
    }                                                                  //
  },                                                                   //
                                                                       //
  updateQuotas: function (token, list, quota) {                        // 377
    checkAuth(token);                                                  // 378
    check(list, String);                                               // 379
    check(quota, Match.OneOf(undefined, null, Number));                // 380
                                                                       //
    if (!list.trim()) {                                                // 382
      throw new Meteor.Error(400, "Must enter addresses.");            // 383
    }                                                                  //
                                                                       //
    var items = list.split("\n");                                      // 386
    var invalid = [];                                                  // 387
    for (var i in babelHelpers.sanitizeForInObject(items)) {           // 388
      var modifier = typeof quota === "number" ? { $set: { quota: quota } } : { $unset: { quota: "" } };
      var n = SignupKeys.update({ email: items[i] }, modifier, { multi: true });
      n += Meteor.users.update({ signupEmail: items[i] }, modifier, { multi: true });
                                                                       //
      if (n < 1) invalid.push(items[i]);                               // 394
    }                                                                  //
                                                                       //
    if (invalid.length > 0) {                                          // 397
      throw new Meteor.Error(404, "These addresses did not map to any user nor invite: " + invalid.join(", "));
    }                                                                  //
  },                                                                   //
                                                                       //
  dismissAdminStatsNotifications: function (token) {                   // 403
    checkAuth(token);                                                  // 404
    globalDb.collections.notifications.remove({ "admin.type": "reportStats" });
  },                                                                   //
                                                                       //
  signUpAsAdmin: function (token) {                                    // 408
    check(token, String);                                              // 409
    checkAuth(token);                                                  // 410
    if (!this.userId) {                                                // 411
      throw new Meteor.Error(403, "Must be logged in to sign up as admin.");
    }                                                                  //
                                                                       //
    if (!Meteor.user().loginIdentities) {                              // 415
      throw new Meteor.Error(403, "Must be logged into an account to sign up as admin.");
    }                                                                  //
                                                                       //
    Meteor.users.update({ _id: this.userId }, { $set: { isAdmin: true, signupKey: "admin" } });
    clearAdminToken(token);                                            // 420
  },                                                                   //
                                                                       //
  redeemSetupToken: function (token) {                                 // 423
    // Redeem an admin token into a setup session.                     //
    check(token, String);                                              // 425
    if (tokenIsValid(token)) {                                         // 426
      var sessId = Random.secret();                                    // 427
      var creationDate = new Date();                                   // 428
      var hashedSessionId = Crypto.createHash("sha256").update(sessId).digest("base64");
      this.connection.sandstormDb.collections.setupSession.upsert({    // 430
        _id: "current-session"                                         // 431
      }, {                                                             //
        creationDate: creationDate,                                    // 433
        hashedSessionId: hashedSessionId                               // 434
      });                                                              //
      // Then, invalidate the token, so one one else can use it.       //
      Fs.unlinkSync(SANDSTORM_ADMIN_TOKEN);                            // 437
      return sessId;                                                   // 438
    } else {                                                           //
      throw new Meteor.Error(401, "Invalid setup token");              // 440
    }                                                                  //
  }                                                                    //
});                                                                    //
                                                                       //
var authorizedAsAdmin = function (token, userId) {                     // 445
  return Match.test(token, Match.OneOf(undefined, null, String)) && (userId && isAdminById(userId) || tokenIsValid(token) || tokenIsSetupSession(token));
};                                                                     //
                                                                       //
Meteor.publish("admin", function (token) {                             // 450
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 451
  return Settings.find();                                              // 452
});                                                                    //
                                                                       //
Meteor.publish("adminServiceConfiguration", function (token) {         // 455
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 456
  return Package["service-configuration"].ServiceConfiguration.configurations.find();
});                                                                    //
                                                                       //
Meteor.publish("publicAdminSettings", function () {                    // 460
  return Settings.find({ _id: { $in: publicAdminSettings } });         // 461
});                                                                    //
                                                                       //
Meteor.publish("adminToken", function (token) {                        // 464
  check(token, String);                                                // 465
  this.added("adminToken", "adminToken", { tokenIsValid: tokenIsValid(token) || tokenIsSetupSession(token) });
  this.ready();                                                        // 467
});                                                                    //
                                                                       //
Meteor.publish("allUsers", function (token) {                          // 470
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 471
  return Meteor.users.find();                                          // 472
});                                                                    //
                                                                       //
Meteor.publish("activityStats", function (token) {                     // 475
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 476
  return ActivityStats.find();                                         // 477
});                                                                    //
                                                                       //
Meteor.publish("statsTokens", function (token) {                       // 480
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 481
  return StatsTokens.find();                                           // 482
});                                                                    //
                                                                       //
Meteor.publish("allPackages", function (token) {                       // 485
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 486
  return Packages.find({ manifest: { $exists: true } }, { fields: { appId: 1, "manifest.appVersion": 1,
      "manifest.actions": 1, "manifest.appTitle": 1 } });              // 489
});                                                                    //
                                                                       //
Meteor.publish("realTimeStats", function (token) {                     // 492
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 493
                                                                       //
  // Last five minutes.                                                //
  this.added("realTimeStats", "now", computeStats(new Date(Date.now() - 5 * 60 * 1000)));
                                                                       //
  // Since last sample.                                                //
  var lastSample = ActivityStats.findOne({}, { sort: { timestamp: -1 } });
  var lastSampleTime = lastSample ? lastSample.timestamp : new Date(0);
  this.added("realTimeStats", "today", computeStats(lastSampleTime));  // 501
                                                                       //
  // TODO(someday): Update every few minutes?                          //
                                                                       //
  this.ready();                                                        // 505
});                                                                    //
                                                                       //
Meteor.publish("adminLog", function (token) {                          // 508
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 509
                                                                       //
  var logfile = SANDSTORM_LOGDIR + "/sandstorm.log";                   // 511
                                                                       //
  var fd = Fs.openSync(logfile, "r");                                  // 513
  var startSize = Fs.fstatSync(fd).size;                               // 514
                                                                       //
  // Start tailing at EOF - 8k.                                        //
  var offset = Math.max(0, startSize - 8192);                          // 517
                                                                       //
  var _this = this;                                                    // 519
  function doTail() {                                                  // 520
    for (;;) {                                                         // 521
      var buf = new Buffer(Math.max(1024, startSize - offset));        // 522
      var n = Fs.readSync(fd, buf, 0, buf.length, offset);             // 523
      if (n <= 0) break;                                               // 524
      _this.added("adminLog", offset, { text: buf.toString("utf8", 0, n) });
      offset += n;                                                     // 526
    }                                                                  //
  }                                                                    //
                                                                       //
  // Watch the file for changes.                                       //
  var watcher = Fs.watch(logfile, { persistent: false }, Meteor.bindEnvironment(doTail));
                                                                       //
  // When the subscription stops, stop watching the file.              //
  this.onStop(function () {                                            // 534
    watcher.close();                                                   // 535
    Fs.closeSync(fd);                                                  // 536
  });                                                                  //
                                                                       //
  // Read initial 8k tail data immediately.                            //
  doTail();                                                            // 540
                                                                       //
  // Notify ready.                                                     //
  this.ready();                                                        // 543
});                                                                    //
                                                                       //
Meteor.publish("adminApiTokens", function (token) {                    // 546
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 547
  return ApiTokens.find({                                              // 548
    $or: [{ "frontendRef.ipNetwork": { $exists: true } }, { "frontendRef.ipInterface": { $exists: true } }]
  }, {                                                                 //
    fields: {                                                          // 554
      frontendRef: 1,                                                  // 555
      created: 1,                                                      // 556
      requirements: 1,                                                 // 557
      revoked: 1                                                       // 558
    }                                                                  //
  });                                                                  //
});                                                                    //
                                                                       //
Meteor.publish("featureKey", function (forAdmin, token) {              // 563
  var _this2 = this;                                                   //
                                                                       //
  if (forAdmin && !authorizedAsAdmin(token, this.userId)) return [];   // 564
                                                                       //
  var fields = forAdmin ? FEATURE_KEY_FIELDS_PUBLISHED_TO_ADMINS : PUBLIC_FEATURE_KEY_FIELDS;
                                                                       //
  var db = this.connection.sandstormDb;                                // 569
  var featureKeyQuery = db.collections.featureKey.find({ _id: "currentFeatureKey" });
  var observeHandle = featureKeyQuery.observe({                        // 571
    added: function (doc) {                                            // 572
      // Load and verify the signed feature key.                       //
      var buf = new Buffer(doc.value);                                 // 574
      var featureKey = loadSignedFeatureKey(buf);                      // 575
                                                                       //
      if (featureKey) {                                                // 577
        var _ref;                                                      //
                                                                       //
        // If the signature is valid, publish the feature key information.
        var filteredFeatureKey = (_ref = _).pick.apply(_ref, [featureKey].concat(fields));
        _this2.added("featureKey", doc._id, filteredFeatureKey);       // 580
      }                                                                //
    },                                                                 //
                                                                       //
    changed: function (newDoc, oldDoc) {                               // 584
      // Load and reverify the new signed feature key.                 //
      var buf = new Buffer(newDoc.value);                              // 586
      var featureKey = loadSignedFeatureKey(buf);                      // 587
                                                                       //
      if (featureKey) {                                                // 589
        var _ref2;                                                     //
                                                                       //
        // If the signature is valid, call this.changed() with the interesting fields.
        var filteredFeatureKey = (_ref2 = _).pick.apply(_ref2, [featureKey].concat(fields));
        _this2.changed("featureKey", newDoc._id, filteredFeatureKey);  // 592
      } else {                                                         //
        // Otherwise, call this.removed(), since the new feature key is invalid.
        _this2.removed("featureKey", oldDoc._id);                      // 595
      }                                                                //
    },                                                                 //
                                                                       //
    removed: function (oldDoc) {                                       // 599
      _this2.removed("featureKey", oldDoc._id);                        // 600
    }                                                                  //
  });                                                                  //
                                                                       //
  this.onStop(function () {                                            // 604
    observeHandle.stop();                                              // 605
  });                                                                  //
                                                                       //
  this.ready();                                                        // 608
});                                                                    //
                                                                       //
Meteor.publish("hasAdmin", function (token) {                          // 611
  var _this3 = this;                                                   //
                                                                       //
  // Like hasUsers, but for admins, and with token auth required.      //
  if (!authorizedAsAdmin(token, this.userId)) return [];               // 613
                                                                       //
  // Query if there are any admin users.                               //
  var cursor = Meteor.users.find({ isAdmin: true });                   // 616
  if (cursor.count() > 0) {                                            // 617
    this.added("hasAdmin", "hasAdmin", { hasAdmin: true });            // 618
  } else {                                                             //
    (function () {                                                     //
      var handle = cursor.observeChanges({                             // 620
        added: function (id) {                                         // 621
          _this3.added("hasAdmin", "hasAdmin", { hasAdmin: true });    // 622
          handle.stop();                                               // 623
          handle = null;                                               // 624
        }                                                              //
      });                                                              //
      _this3.onStop(function () {                                      // 627
        if (handle) handle.stop();                                     // 628
      });                                                              //
    })();                                                              //
  }                                                                    //
                                                                       //
  this.ready();                                                        // 632
});                                                                    //
                                                                       //
function observeOauthService(name) {                                   // 635
  Settings.find({ _id: name, value: true }).observe({                  // 636
    added: function () {                                               // 637
      // Tell the oauth library it should accept login attempts from this service.
      Accounts.oauth.registerService(name);                            // 639
    },                                                                 //
                                                                       //
    removed: function () {                                             // 642
      // Tell the oauth library it should deny login attempts from this service.
      Accounts.oauth.unregisterService(name);                          // 644
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
observeOauthService("github");                                         // 649
observeOauthService("google");                                         // 650
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=admin-server.js.map
