(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/grain-server.js                                              //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var Crypto = Npm.require("crypto");                                    // 17
                                                                       //
var emailLinkWithInlineStyle = function (url, text) {                  // 19
  return "<a href='" + url + "' style='display:inline-block;text-decoration:none;" + "font-family:sans-serif;width:200px;min-height:30px;line-height:30px;" + "border-radius:4px;text-align:center;background:#762F87;color:white'>" + text + "</a>";
};                                                                     //
                                                                       //
Meteor.publish("grainTopBar", function (grainId) {                     // 26
  check(grainId, String);                                              // 27
                                                                       //
  var result = [Grains.find({                                          // 29
    _id: grainId,                                                      // 31
    $or: [{ userId: this.userId }, { "private": { $ne: true } }]       // 32
  }, {                                                                 //
    fields: {                                                          // 37
      title: 1,                                                        // 38
      userId: 1,                                                       // 39
      identityId: 1,                                                   // 40
      "private": 1                                                     // 41
    }                                                                  //
  })];                                                                 //
  if (this.userId) {                                                   // 45
    var myIdentityIds = SandstormDb.getUserIdentityIds(globalDb.getUser(this.userId));
    result.push(ApiTokens.find({                                       // 47
      grainId: grainId,                                                // 48
      $or: [{ "owner.user.identityId": { $in: myIdentityIds } }, { identityId: { $in: myIdentityIds } }]
    }));                                                               //
  }                                                                    //
                                                                       //
  return result;                                                       // 56
});                                                                    //
                                                                       //
// We allow users to learn package information about a grain they own.
// This is used for obtaining icon and app title information for grains
// you own, which is used in the sidebar. It is not a security/privacy
// risk since it only exposes this information for grains the user owns.
Meteor.publish("packageByGrainId", function (grainId) {                // 63
  check(grainId, String);                                              // 64
  var publishThis = [];                                                // 65
  // We need to publish the packageId so that client-side code can     //
  // find the right package.                                           //
  var thisGrainCursor = Grains.find({                                  // 68
    _id: grainId,                                                      // 69
    userId: this.userId                                                // 70
  }, {                                                                 //
    fields: { packageId: 1 }                                           // 72
  });                                                                  //
  publishThis.push(thisGrainCursor);                                   // 74
                                                                       //
  if (thisGrainCursor.count()) {                                       // 76
    var thisGrain = thisGrainCursor.fetch()[0];                        // 77
    var thisPackageCursor = Packages.find({ _id: thisGrain.packageId });
    publishThis.push(thisPackageCursor);                               // 79
  }                                                                    //
                                                                       //
  return publishThis;                                                  // 82
});                                                                    //
                                                                       //
Meteor.publish("tokenInfo", function (token) {                         // 85
  // Allows the client side to map a raw token to its entry in ApiTokens, and the additional
  // metadata that it will need to display the app icon and title.  We do not care about making
  // the metadata reactive.                                            //
  check(token, String);                                                // 89
                                                                       //
  var hashedToken = Crypto.createHash("sha256").update(token).digest("base64");
  var apiToken = ApiTokens.findOne({                                   // 92
    _id: hashedToken                                                   // 93
  }, {                                                                 //
    fields: {                                                          // 95
      grainId: 1,                                                      // 96
      identityId: 1,                                                   // 97
      owner: 1,                                                        // 98
      revoked: 1                                                       // 99
    }                                                                  //
  });                                                                  //
  if (!apiToken) {                                                     // 102
    this.added("tokenInfo", token, { invalidToken: true });            // 103
  } else if (apiToken.revoked) {                                       //
    this.added("tokenInfo", token, { revoked: true });                 // 105
  } else {                                                             //
    var grainId = apiToken.grainId;                                    // 107
    var grain = Grains.findOne({                                       // 108
      _id: grainId                                                     // 109
    }, {                                                               //
      fields: {                                                        // 111
        packageId: 1,                                                  // 112
        appId: 1,                                                      // 113
        userId: 1                                                      // 114
      }                                                                //
    });                                                                //
    if (!grain) {                                                      // 117
      this.added("tokenInfo", token, { invalidToken: true });          // 118
    } else {                                                           //
      if (apiToken.owner && apiToken.owner.user) {                     // 120
        var identity = globalDb.getIdentity(apiToken.owner.user.identityId);
        var metadata = apiToken.owner.user.denormalizedGrainMetadata;  // 122
        if (identity && metadata) {                                    // 123
          this.added("tokenInfo", token, { identityOwner: _.pick(identity, "_id", "profile"),
            grainId: grainId,                                          // 126
            grainMetadata: metadata });                                // 127
        } else {                                                       //
          this.added("tokenInfo", token, { invalidToken: true });      // 129
        }                                                              //
      } else if (!apiToken.owner || "webkey" in apiToken.owner) {      //
        if (this.userId) {                                             // 132
          var user = Meteor.users.findOne({ _id: this.userId });       // 133
          var identityIds = SandstormDb.getUserIdentityIds(user);      // 134
          var childToken = ApiTokens.findOne({ "owner.user.identityId": { $in: identityIds },
            parentToken: apiToken._id });                              // 136
          if (childToken || this.userId === grain.userId || identityIds.indexOf(apiToken.identityId) >= 0) {
            this.added("tokenInfo", token, { alreadyRedeemed: true, grainId: apiToken.grainId });
            this.ready();                                              // 140
            return;                                                    // 141
          }                                                            //
        }                                                              //
                                                                       //
        var pkg = Packages.findOne({ _id: grain.packageId }, { fields: { manifest: 1 } });
        var appTitle = pkg && pkg.manifest && pkg.manifest.appTitle || { defaultText: "" };
        var appIcon = undefined;                                       // 147
        if (pkg && pkg.manifest && pkg.manifest.metadata && pkg.manifest.metadata.icons) {
          var icons = pkg.manifest.metadata.icons;                     // 149
          appIcon = icons.grain || icons.appGrid;                      // 150
        }                                                              //
                                                                       //
        var denormalizedGrainMetadata = {                              // 153
          appTitle: appTitle,                                          // 154
          icon: appIcon,                                               // 155
          appId: appIcon ? undefined : grain.appId                     // 156
        };                                                             //
        this.added("tokenInfo", token, { webkey: true,                 // 158
          grainId: grainId,                                            // 160
          grainMetadata: denormalizedGrainMetadata });                 // 161
      } else {                                                         //
        this.added("tokenInfo", token, { invalidToken: true });        // 163
      }                                                                //
    }                                                                  //
  }                                                                    //
                                                                       //
  this.ready();                                                        // 168
  return;                                                              // 169
});                                                                    //
                                                                       //
Meteor.publish("requestingAccess", function (grainId) {                // 172
  check(grainId, String);                                              // 173
                                                                       //
  if (!this.userId) {                                                  // 175
    throw new Meteor.Error(403, "Must be logged in to request access.");
  }                                                                    //
                                                                       //
  var identityIds = SandstormDb.getUserIdentityIds(Meteor.users.findOne({ _id: this.userId }));
                                                                       //
  var grain = globalDb.getGrain(grainId);                              // 181
  if (!grain) {                                                        // 182
    throw new Meteor.Error(404, "Grain not found.");                   // 183
  }                                                                    //
                                                                       //
  if (grain.userId === this.userId) {                                  // 186
    this.added("grantedAccessRequests", Random.id(), { grainId: grainId, identityId: grain.identityId });
  }                                                                    //
                                                                       //
  var _this = this;                                                    // 191
  var query = ApiTokens.find({ grainId: grainId, accountId: grain.userId,
    parentToken: { $exists: false },                                   // 193
    "owner.user.identityId": { $in: identityIds },                     // 194
    revoked: { $ne: true } });                                         // 195
  var handle = query.observe({                                         // 196
    added: function (apiToken) {                                       // 197
      _this.added("grantedAccessRequests", Random.id(), { grainId: grainId, identityId: apiToken.owner.user.identityId });
    }                                                                  //
  });                                                                  //
                                                                       //
  this.onStop(function () {                                            // 203
    return handle.stop();                                              //
  });                                                                  //
});                                                                    //
                                                                       //
Meteor.publish("grainSize", function (grainId) {                       // 206
  // Publish pseudo-collection containing the size of the grain opened in the given session.
  check(grainId, String);                                              // 208
                                                                       //
  var grain = Grains.findOne(grainId);                                 // 210
  if (!grain || grain.userId !== this.userId) {                        // 211
    return [];                                                         // 212
  }                                                                    //
                                                                       //
  var supervisor = globalBackend.cap().getGrain(this.userId, grainId).supervisor;
                                                                       //
  var _this = this;                                                    // 217
  var stopped = false;                                                 // 218
  var promise = getGrainSize(supervisor);                              // 219
                                                                       //
  function getNext(oldSize) {                                          // 221
    promise = getGrainSize(supervisor, oldSize);                       // 222
    promise.then(function (size) {                                     // 223
      if (!stopped) {                                                  // 224
        if (size !== oldSize) {                                        // 225
          // sometimes there are false alarms                          //
          _this.changed("grainSizes", grainId, { size: size });        // 226
        }                                                              //
                                                                       //
        getNext(size);                                                 // 229
      }                                                                //
    }, function (err) {                                                //
                                                                       //
      if (!stopped) {                                                  // 233
        if (err.kjType === "disconnected") {                           // 234
          _this.stop();                                                // 235
        } else {                                                       //
          _this.error(err);                                            // 237
        }                                                              //
      }                                                                //
    });                                                                //
  }                                                                    //
                                                                       //
  promise.then(function (size) {                                       // 243
    if (!stopped) {                                                    // 244
      _this.added("grainSizes", grainId, { size: size });              // 245
      _this.ready();                                                   // 246
      getNext(size);                                                   // 247
    }                                                                  //
  }, function (err) {                                                  //
                                                                       //
    if (!stopped) {                                                    // 251
      if (err.kjType === "disconnected") {                             // 252
        _this.stop();                                                  // 253
      } else {                                                         //
        _this.error(err);                                              // 255
      }                                                                //
    }                                                                  //
  });                                                                  //
                                                                       //
  _this.onStop(function () {                                           // 260
    stopped = true;                                                    // 261
    promise.cancel();                                                  // 262
  });                                                                  //
});                                                                    //
                                                                       //
Meteor.methods({                                                       // 266
  deleteGrain: function (grainId) {                                    // 267
    check(grainId, String);                                            // 268
                                                                       //
    if (this.userId) {                                                 // 270
      var grain = Grains.findOne({ _id: grainId, userId: this.userId });
      if (grain) {                                                     // 272
        Grains.remove(grainId);                                        // 273
        globalDb.removeApiTokens({                                     // 274
          grainId: grainId,                                            // 275
          $or: [{ owner: { $exists: false } }, { owner: { webkey: null } }]
        });                                                            //
        if (grain.lastUsed) {                                          // 281
          DeleteStats.insert({ type: isDemoUser() ? "demoGrain" : "grain",
            lastActive: grain.lastUsed, appId: grain.appId });         // 283
        }                                                              //
                                                                       //
        if (!this.isSimulation) {                                      // 286
          waitPromise(globalBackend.deleteGrain(grainId, this.userId));
          Meteor.call("deleteUnusedPackages", grain.appId);            // 288
        }                                                              //
      }                                                                //
    }                                                                  //
  },                                                                   //
                                                                       //
  forgetGrain: function (grainId, identityId) {                        // 294
    // TODO(cleanup): For now we are ignoring `identityId`, but maybe we should expose finer-grained
    //  forgetting.                                                    //
                                                                       //
    check(grainId, String);                                            // 298
    check(identityId, String);                                         // 299
                                                                       //
    if (!this.userId) {                                                // 301
      throw new Meteor.Error(403, "Must be logged in to forget a grain.");
    }                                                                  //
                                                                       //
    SandstormDb.getUserIdentityIds(Meteor.user()).forEach(function (identityId) {
      globalDb.removeApiTokens({ grainId: grainId, "owner.user.identityId": identityId });
    });                                                                //
  },                                                                   //
                                                                       //
  updateGrainPreferredIdentity: function (grainId, identityId) {       // 310
    check(grainId, String);                                            // 311
    check(identityId, String);                                         // 312
    if (!this.userId) {                                                // 313
      throw new Meteor.Error(403, "Must be logged in.");               // 314
    }                                                                  //
                                                                       //
    var grain = globalDb.getGrain(grainId) || {};                      // 317
    if (!grain.userId === this.userId) {                               // 318
      throw new Meteor.Error(403, "Grain not owned by current user.");
    }                                                                  //
                                                                       //
    Grains.update({ _id: grainId }, { $set: { identityId: identityId } });
  },                                                                   //
                                                                       //
  updateGrainTitle: function (grainId, newTitle, identityId) {         // 325
    check(grainId, String);                                            // 326
    check(newTitle, String);                                           // 327
    check(identityId, String);                                         // 328
    if (this.userId) {                                                 // 329
      var grain = Grains.findOne(grainId);                             // 330
      if (grain) {                                                     // 331
        if (grain.userId === this.userId) {                            // 332
          Grains.update({ _id: grainId, userId: this.userId }, { $set: { title: newTitle } });
        } else {                                                       //
          if (!globalDb.userHasIdentity(this.userId, identityId)) {    // 335
            throw new Meteor.Error(403, "Current user does not have identity " + identityId);
          }                                                            //
                                                                       //
          var token = ApiTokens.findOne({                              // 339
            grainId: grainId,                                          // 340
            objectId: { $exists: false },                              // 341
            "owner.user.identityId": identityId                        // 342
          }, {                                                         //
            sort: { created: 1 }                                       // 344
          });                                                          //
          if (token) {                                                 // 346
            ApiTokens.update(token._id, { $set: { "owner.user.title": newTitle } });
          }                                                            //
        }                                                              //
      }                                                                //
    }                                                                  //
  },                                                                   //
                                                                       //
  privatizeGrain: function (grainId) {                                 // 354
    check(grainId, String);                                            // 355
    if (this.userId) {                                                 // 356
      Grains.update({ _id: grainId, userId: this.userId }, { $set: { "private": true } });
    }                                                                  //
  },                                                                   //
                                                                       //
  inviteUsersToGrain: function (origin, identityId, grainId, title, roleAssignment, contacts, message) {
    var _this2 = this;                                                 //
                                                                       //
    if (!this.isSimulation) {                                          // 363
      var _ret = (function () {                                        //
        check(origin, String);                                         // 364
        check(identityId, String);                                     // 365
        check(grainId, String);                                        // 366
        check(title, String);                                          // 367
        check(roleAssignment, roleAssignmentPattern);                  // 368
        check(contacts, [{                                             // 369
          _id: String,                                                 // 371
          isDefault: Match.Optional(Boolean),                          // 372
          profile: Match.ObjectIncluding({                             // 373
            service: String,                                           // 374
            name: String,                                              // 375
            intrinsicName: String                                      // 376
          })                                                           //
        }]);                                                           //
        check(message, { text: String, html: String });                // 380
        if (!_this2.userId) {                                          // 381
          throw new Meteor.Error(403, "Must be logged in to share by email.");
        }                                                              //
                                                                       //
        if (!globalDb.userHasIdentity(_this2.userId, identityId)) {    // 385
          throw new Meteor.Error(403, "Not an identity of the current user: " + identityId);
        }                                                              //
                                                                       //
        if (contacts.length === 0) {                                   // 389
          throw new Meteor.Error(400, "No contacts were provided.");   // 390
        }                                                              //
                                                                       //
        var accountId = _this2.userId;                                 // 393
        var outerResult = { successes: [], failures: [] };             // 394
        var fromEmail = globalDb.getReturnAddressWithDisplayName(identityId);
        var replyTo = globalDb.getPrimaryEmail(accountId, identityId);
        contacts.forEach(function (contact) {                          // 397
          if (contact.isDefault && contact.profile.service === "email") {
            var emailAddress = contact.profile.intrinsicName;          // 399
            var result = SandstormPermissions.createNewApiToken(globalDb, { identityId: identityId, accountId: accountId }, grainId, "email invitation for " + emailAddress, roleAssignment, { webkey: { forSharing: true } });
            var url = origin + "/shared/" + result.token;              // 404
            var html = message.html + "<br><br>" + emailLinkWithInlineStyle(url, "Open Shared Grain") + "<div style='font-size:8pt;font-style:italic;color:gray'>" + "Note: If you forward this email to other people, they will be able to access " + "the share as well. To prevent this, remove the button before forwarding.</div>";
            try {                                                      // 410
              SandstormEmail.send({                                    // 411
                to: emailAddress,                                      // 412
                from: fromEmail,                                       // 413
                replyTo: replyTo,                                      // 414
                subject: title + " - Invitation to collaborate",       // 415
                text: message.text + "\n\nFollow this link to open the shared grain:\n\n" + url + "\n\nNote: If you forward this email to other people, they will be able to " + "access the share as well. To prevent this, remove the link before forwarding.",
                html: html                                             // 419
              });                                                      //
            } catch (e) {                                              //
              outerResult.failures.push({ contact: contact, error: e.toString() });
            }                                                          //
          } else {                                                     //
            var result = SandstormPermissions.createNewApiToken(globalDb, { identityId: identityId, accountId: accountId }, grainId, "direct invitation to " + contact.profile.intrinsicName, roleAssignment, { user: { identityId: contact._id, title: title } });
            var url = origin + "/shared/" + result.token;              // 429
            try {                                                      // 430
              var identity = Meteor.users.findOne({ _id: contact._id });
              var email = _.findWhere(SandstormDb.getVerifiedEmails(identity), { primary: true });
              if (email) {                                             // 434
                var intrinsicName = contact.profile.intrinsicName;     // 435
                var loginNote = undefined;                             // 436
                if (contact.profile.service === "google") {            // 437
                  loginNote = "Google account with address " + email.email;
                } else if (contact.profile.service === "github") {     //
                  loginNote = "Github account with username " + intrinsicName;
                } else if (contact.profile.service === "email") {      //
                  loginNote = "email address " + intrinsicName;        // 442
                } else {                                               //
                  throw new Meteor.Error(500, "Unknown service to email share link.");
                }                                                      //
                                                                       //
                var html = message.html + "<br><br>" + emailLinkWithInlineStyle(url, "Open Shared Grain") + "<div style='font-size:8pt;font-style:italic;color:gray'>" + "Note: You will need to log in with your " + loginNote + " to access this grain.";
                SandstormEmail.send({                                  // 452
                  to: email.email,                                     // 453
                  from: fromEmail,                                     // 454
                  replyTo: replyTo,                                    // 455
                  subject: title + " - Invitation to collaborate",     // 456
                  text: message.text + "\n\nFollow this link to open the shared grain:\n\n" + url + "\n\nNote: You will need to log in with your " + loginNote + " to access this grain.",
                  html: html                                           // 460
                });                                                    //
              } else {                                                 //
                outerResult.failures.push({ contact: contact, warning: "User does not have a " + "verified email, so notification of this share was not sent to them. Please " + "manually share " + url + " with them." });
              }                                                        //
            } catch (e) {                                              //
              outerResult.failures.push({ contact: contact, error: e.toString(),
                warning: "Share succeeded, but there was an error emailing the user. Please " + "manually share " + url + " with them." });
            }                                                          //
          }                                                            //
        });                                                            //
                                                                       //
        return {                                                       // 475
          v: outerResult                                               //
        };                                                             //
      })();                                                            //
                                                                       //
      if (typeof _ret === "object") return _ret.v;                     //
    }                                                                  //
  },                                                                   //
                                                                       //
  requestAccess: function (origin, grainId, identityId) {              // 479
    check(origin, String);                                             // 480
    check(grainId, String);                                            // 481
    check(identityId, String);                                         // 482
    if (!this.isSimulation) {                                          // 483
      if (!this.userId) {                                              // 484
        throw new Meteor.Error(403, "Must be logged in to request access.");
      }                                                                //
                                                                       //
      if (!globalDb.userHasIdentity(this.userId, identityId)) {        // 488
        throw new Meteor.Error(403, "Not an identity of the current user: " + identityId);
      }                                                                //
                                                                       //
      var grain = Grains.findOne(grainId);                             // 492
      if (!grain) {                                                    // 493
        throw new Meteor.Error(404, "No such grain");                  // 494
      }                                                                //
                                                                       //
      var grainOwner = globalDb.getUser(grain.userId);                 // 497
      var email = _.findWhere(SandstormDb.getUserEmails(grainOwner), { primary: true });
      if (!email) {                                                    // 499
        throw new Meteor.Error("no email", "Grain owner has no email address.");
      }                                                                //
                                                                       //
      var emailAddress = email.email;                                  // 503
                                                                       //
      var identity = globalDb.getIdentity(identityId);                 // 505
      globalDb.addContact(grainOwner._id, identityId);                 // 506
                                                                       //
      var fromEmail = globalDb.getReturnAddressWithDisplayName(identityId);
      var replyTo = globalDb.getPrimaryEmail(Meteor.userId(), identityId);
                                                                       //
      // TODO(soon): In the HTML version, we should display an identity card.
      var identityNote = "";                                           // 512
      if (identity.profile.service === "google") {                     // 513
        identityNote = " (" + identity.privateIntrinsicName + ")";     // 514
      } else if (identity.profile.service === "github") {              //
        identityNote = " (" + identity.profile.intrinsicName + " on GitHub)";
      } else if (identity.profile.service === "email") {               //
        identityNote = " (" + identity.profile.intrinsicName + ")";    // 518
      }                                                                //
                                                                       //
      var message = identity.profile.name + identityNote + " is requesting access to your grain: " + grain.title + ".";
                                                                       //
      var url = origin + "/share/" + grainId + "/" + identityId;       // 524
                                                                       //
      var html = message + "<br><br>" + emailLinkWithInlineStyle(url, "Open Sharing Menu");
                                                                       //
      var user = Meteor.user();                                        // 529
      var ACCESS_REQUEST_LIMIT = 10;                                   // 530
      var resetCount = true;                                           // 531
      if (user.accessRequests) {                                       // 532
        if (user.accessRequests.resetOn < new Date()) {                // 533
          Meteor.users.update({ _id: user._id }, { $unset: { accessRequests: 1 } });
        } else if (user.accessRequests.count >= ACCESS_REQUEST_LIMIT) {
          throw new Meteor.Error(403, "For spam control reasons, you are not allowed to make " + "more than " + ACCESS_REQUEST_LIMIT + " access requests per day.");
        } else {                                                       //
          resetCount = false;                                          // 540
        }                                                              //
      }                                                                //
                                                                       //
      var modifier = { $inc: { "accessRequests.count": 1 } };          // 544
      if (resetCount) {                                                // 545
        var tomorrow = new Date(Date.now() + 86400000);                // 546
        modifier.$set = { "accessRequests.resetOn": tomorrow };        // 547
      }                                                                //
                                                                       //
      Meteor.users.update({ _id: user._id }, modifier);                // 550
                                                                       //
      SandstormEmail.send({                                            // 552
        to: emailAddress,                                              // 553
        from: fromEmail,                                               // 554
        replyTo: replyTo,                                              // 555
        subject: grain.title + " - Request for access",                // 556
        text: message + "\n\nFollow this link to share access:\n\n" + url,
        html: html                                                     // 558
      });                                                              //
    }                                                                  //
  }                                                                    //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=grain-server.js.map
