(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var MeteorWrapperLdapjs = Package['typ:ldapjs'].MeteorWrapperLdapjs;
var Accounts = Package['accounts-base'].Accounts;
var AccountsServer = Package['accounts-base'].AccountsServer;
var check = Package.check.check;
var Match = Package.check.Match;
var _ = Package.underscore._;
var ECMAScript = Package.ecmascript.ECMAScript;
var babelHelpers = Package['babel-runtime'].babelHelpers;
var Symbol = Package['ecmascript-runtime'].Symbol;
var Map = Package['ecmascript-runtime'].Map;
var Set = Package['ecmascript-runtime'].Set;
var Promise = Package.promise.Promise;

/* Package-scope variables */
var Future, LDAP_DEFAULTS, LDAP;

(function(){

///////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                       //
// packages/accounts-ldap/ldap_server.js                                                                 //
//                                                                                                       //
///////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                         //
Future = Npm.require("fibers/future");                                                                   // 1
                                                                                                         //
// At a minimum, set up LDAP_DEFAULTS.url and .dn according to                                           //
// your needs. url should appear as 'ldap://your.url.here'                                               //
// dn should appear in normal ldap format of comma separated attribute=value                             //
// e.g. 'uid=someuser,cn=users,dc=somevalue'                                                             //
LDAP_DEFAULTS = {                                                                                        // 7
  url: false,                                                                                            // 8
  port: "389",                                                                                           // 9
  dn: false,                                                                                             // 10
  searchDN: false,                                                                                       // 11
  searchCredentials: false,                                                                              // 12
  createNewUser: true,                                                                                   // 13
  defaultDomain: false,                                                                                  // 14
  searchResultsProfileMap: false,                                                                        // 15
  base: null,                                                                                            // 16
  search: "(objectclass=*)",                                                                             // 17
  ldapsCertificate: false                                                                                // 18
};                                                                                                       //
                                                                                                         //
/**                                                                                                      //
 @class LDAP                                                                                             //
 @constructor                                                                                            //
 */                                                                                                      //
var LDAP = function () {                                                                                 // 25
  // Set options                                                                                         //
  this.options = _.clone(LDAP_DEFAULTS);                                                                 // 27
                                                                                                         //
  // Because NPM ldapjs module has some binary builds,                                                   //
  // We had to create a wraper package for it and build for                                              //
  // certain architectures. The package typ:ldap-js exports                                              //
  // 'MeteorWrapperLdapjs' which is a wrapper for the npm module                                         //
  this.ldapjs = MeteorWrapperLdapjs;                                                                     // 33
};                                                                                                       //
                                                                                                         //
/**                                                                                                      //
 * Attempt to bind (authenticate) ldap                                                                   //
 * and perform a dn search if specified                                                                  //
 *                                                                                                       //
 * @method ldapCheck                                                                                     //
 *                                                                                                       //
 * @param {Object} options  Object with username, ldapPass and overrides for LDAP_DEFAULTS object.       //
 * Additionally the searchBeforeBind parameter can be specified, which is used to search for the DN      //
 * if not provided.                                                                                      //
 */                                                                                                      //
LDAP.prototype.ldapCheck = function (db, options) {                                                      // 46
                                                                                                         //
  var _this = this;                                                                                      // 48
                                                                                                         //
  options = options || {};                                                                               // 50
                                                                                                         //
  if (options.hasOwnProperty("username") && options.hasOwnProperty("ldapPass")) {                        // 52
    var _ret = (function () {                                                                            //
      _this.options.base = db.getLdapBase();                                                             // 53
      _this.options.searchBeforeBind = {};                                                               // 54
      _this.options.searchBeforeBind[db.getLdapSearchUsername()] = options.username;                     // 55
      _this.options.filter = db.getLdapFilter() || "(objectclass=*)";                                    // 56
      _this.options.searchBindDn = db.getLdapSearchBindDn();                                             // 57
      _this.options.searchBindPassword = db.getLdapSearchBindPassword();                                 // 58
                                                                                                         //
      var resolved = false;                                                                              // 60
      var ldapAsyncFut = new Future();                                                                   // 61
                                                                                                         //
      // Create ldap client                                                                              //
      var fullUrl = db.getLdapUrl();                                                                     // 64
      var client = null;                                                                                 // 65
                                                                                                         //
      var errorFunc = function (err) {                                                                   // 67
        if (err) {                                                                                       // 68
          if (resolved) return;                                                                          // 69
          resolved = true;                                                                               // 70
          ldapAsyncFut["return"]({                                                                       // 71
            error: err                                                                                   // 72
          });                                                                                            //
        }                                                                                                //
      };                                                                                                 //
                                                                                                         //
      if (fullUrl.indexOf("ldaps://") === 0) {                                                           // 77
        client = _this.ldapjs.createClient({                                                             // 78
          url: fullUrl,                                                                                  // 79
          tlsOptions: {                                                                                  // 80
            ca: [_this.options.ldapsCertificate]                                                         // 81
          }                                                                                              //
        }, errorFunc);                                                                                   //
      } else {                                                                                           //
        client = _this.ldapjs.createClient({                                                             // 85
          url: fullUrl                                                                                   // 86
        }, errorFunc);                                                                                   //
      }                                                                                                  //
                                                                                                         //
      client.on("error", errorFunc);                                                                     // 90
                                                                                                         //
      // Slide @xyz.whatever from username if it was passed in                                           //
      // and replace it with the domain specified in defaults                                            //
      var emailSliceIndex = options.username.indexOf("@");                                               // 94
      var username = undefined;                                                                          // 95
      var domain = _this.options.defaultDomain;                                                          // 96
                                                                                                         //
      // If user appended email domain, strip it out                                                     //
      // And use the defaults.defaultDomain if set                                                       //
      if (emailSliceIndex !== -1) {                                                                      // 100
        username = options.username.substring(0, emailSliceIndex);                                       // 101
        domain = domain || options.username.substring(emailSliceIndex + 1, options.username.length);     // 102
      } else {                                                                                           //
        username = options.username;                                                                     // 104
      }                                                                                                  //
                                                                                                         //
      if (_this.options.searchBindDn) {                                                                  // 107
        var _ret2 = (function () {                                                                       //
          var ldapBindFut = new Future();                                                                // 108
          client.bind(_this.options.searchBindDn, _this.options.searchBindPassword, function (err) {     // 109
            if (err) {                                                                                   // 111
              ldapBindFut["throw"](err);                                                                 // 112
            } else {                                                                                     //
              ldapBindFut["return"]();                                                                   // 114
            }                                                                                            //
          });                                                                                            //
                                                                                                         //
          try {                                                                                          // 119
            ldapBindFut.wait();                                                                          // 120
          } catch (err) {                                                                                //
            return {                                                                                     // 122
              v: {                                                                                       //
                v: { error: err }                                                                        //
              }                                                                                          //
            };                                                                                           //
          }                                                                                              //
        })();                                                                                            //
                                                                                                         //
        if (typeof _ret2 === "object") return _ret2.v;                                                   //
      }                                                                                                  //
      // initialize result                                                                               //
      var retObject = {                                                                                  // 126
        username: username,                                                                              // 127
        email: domain ? username + "@" + domain : false,                                                 // 128
        emptySearch: true,                                                                               // 129
        searchResults: {}                                                                                // 130
      };                                                                                                 //
                                                                                                         //
      var filter = _this.options.filter;                                                                 // 133
      Object.keys(_this.options.searchBeforeBind).forEach(function (searchKey) {                         // 134
        filter = "&" + filter + "(" + searchKey + "=" + _this.options.searchBeforeBind[searchKey] + ")";
      });                                                                                                //
                                                                                                         //
      var searchOptions = {                                                                              // 138
        scope: "sub",                                                                                    // 139
        sizeLimit: 1,                                                                                    // 140
        filter: filter                                                                                   // 141
      };                                                                                                 //
                                                                                                         //
      // perform LDAP search to determine DN                                                             //
      client.search(_this.options.base, searchOptions, function (err, res) {                             // 145
        if (err) {                                                                                       // 146
          if (resolved) return;                                                                          // 147
          resolved = true;                                                                               // 148
          ldapAsyncFut["return"]({                                                                       // 149
            error: err                                                                                   // 150
          });                                                                                            //
          return;                                                                                        // 152
        }                                                                                                //
                                                                                                         //
        retObject.emptySearch = true;                                                                    // 155
        res.on("searchEntry", function (entry) {                                                         // 156
          retObject.dn = entry.objectName;                                                               // 157
          retObject.username = retObject.dn;                                                             // 158
          retObject.emptySearch = false;                                                                 // 159
                                                                                                         //
          retObject.searchResults = _.omit(entry.object, "userPassword");                                // 161
                                                                                                         //
          // use the determined DN to bind                                                               //
          client.bind(entry.objectName, options.ldapPass, function (err) {                               // 164
            try {                                                                                        // 165
              if (err) {                                                                                 // 166
                throw new Meteor.Error(err.code, err.message);                                           // 167
              } else {                                                                                   //
                resolved = true;                                                                         // 169
                ldapAsyncFut["return"](retObject);                                                       // 170
              }                                                                                          //
            } catch (e) {                                                                                //
              if (resolved) return;                                                                      // 174
              resolved = true;                                                                           // 175
              ldapAsyncFut["return"]({                                                                   // 176
                error: e                                                                                 // 177
              });                                                                                        //
            }                                                                                            //
          });                                                                                            //
        });                                                                                              //
                                                                                                         //
        res.on("error", errorFunc);                                                                      // 183
                                                                                                         //
        // If no dn is found, return as is.                                                              //
        res.on("end", function (result) {                                                                // 186
          if (retObject.dn === undefined) {                                                              // 187
            resolved = true;                                                                             // 188
            ldapAsyncFut["return"](retObject);                                                           // 189
          }                                                                                              //
        });                                                                                              //
      });                                                                                                //
                                                                                                         //
      return {                                                                                           // 194
        v: ldapAsyncFut.wait()                                                                           //
      };                                                                                                 //
    })();                                                                                                //
                                                                                                         //
    if (typeof _ret === "object") return _ret.v;                                                         //
  } else {                                                                                               //
    throw new Meteor.Error(403, "Missing LDAP Auth Parameter");                                          // 197
  }                                                                                                      //
};                                                                                                       //
                                                                                                         //
// Register login handler with Meteor                                                                    //
// Here we create a new LDAP instance with options passed from                                           //
// Meteor.loginWithLDAP on client side                                                                   //
// @param {Object} loginRequest will consist of username, ldapPass, ldap, and ldapOptions                //
Accounts.registerLoginHandler("ldap", function (loginRequest) {                                          // 206
  // If 'ldap' isn't set in loginRequest object,                                                         //
  // then this isn't the proper handler (return undefined)                                               //
  if (!loginRequest.ldap) {                                                                              // 209
    return undefined;                                                                                    // 210
  }                                                                                                      //
                                                                                                         //
  if (!Accounts.identityServices.ldap.isEnabled()) {                                                     // 213
    throw new Meteor.Error(403, "LDAP service is disabled.");                                            // 214
  }                                                                                                      //
                                                                                                         //
  check(loginRequest, {                                                                                  // 217
    ldap: true,                                                                                          // 218
    username: String,                                                                                    // 219
    ldapPass: String                                                                                     // 220
  });                                                                                                    //
                                                                                                         //
  // Instantiate LDAP                                                                                    //
  var ldapObj = new LDAP();                                                                              // 224
                                                                                                         //
  // Call ldapCheck and get response                                                                     //
  var ldapResponse = ldapObj.ldapCheck(this.connection.sandstormDb, loginRequest);                       // 227
                                                                                                         //
  if (ldapResponse.error) {                                                                              // 229
    return {                                                                                             // 230
      userId: null,                                                                                      // 231
      error: ldapResponse.error                                                                          // 232
    };                                                                                                   //
  } else if (ldapResponse.emptySearch) {                                                                 //
    return {                                                                                             // 235
      userId: null,                                                                                      // 236
      error: new Meteor.Error(403, "User not found in LDAP")                                             // 237
    };                                                                                                   //
  } else {                                                                                               //
    // Set initial userId and token vals                                                                 //
    return Accounts.updateOrCreateUserFromExternalService("ldap", { id: ldapResponse.dn, username: loginRequest.username,
      rawAttrs: ldapResponse.searchResults }, {});                                                       // 243
  }                                                                                                      //
});                                                                                                      //
///////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['accounts-ldap'] = {
  LDAP: LDAP,
  LDAP_DEFAULTS: LDAP_DEFAULTS
};

})();

//# sourceMappingURL=accounts-ldap.js.map
